@@ -10,6 +10,8 @@ struct lfo {
 	int phase;
 };
 
+LIBXMP_BEGIN_DECLS
+
 int  libxmp_lfo_get(struct context_data *, struct lfo *, int);
 void libxmp_lfo_update(struct lfo *);
 void libxmp_lfo_set_phase(struct lfo *, int);
@@ -17,4 +19,6 @@ void libxmp_lfo_set_depth(struct lfo *, int);
 void libxmp_lfo_set_rate(struct lfo *, int);
 void libxmp_lfo_set_waveform(struct lfo *, int);
 
+LIBXMP_END_DECLS
+
 #endif
