@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2021 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2026 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -177,7 +177,7 @@ static int c669_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	xxs->flg = xxs->lpe ? XMP_SAMPLE_LOOP : 0;	/* 1 == Forward loop */
 
 	sub->vol = 0x40;
-	sub->pan = 0x80;
+	sub->pan = XMP_INST_NO_DEFAULT_PAN;
 	sub->sid = i;
 
 	if (xxs->len > 0)
