@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2025 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2026 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -360,13 +360,24 @@ static int far_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		vol  = *pos++;
 		fxb  = *pos++;
 
-		if (note)
+		/* The instrument number is completely ignored if no note is
+		 * present; whatever note is playing will continue. */
+		if (note) {
 		    event->note = note + 48;
-		if (event->note || ins)
 		    event->ins = ins + 1;
+		}
 
-		if (vol >= 0x01 && vol <= 0x10)
+		if (vol >= 0x01 && vol <= 0x10) {
 		    event->vol = (vol - 1) * 16 + 1;
+		} else if (note || vol) {
+		    /* aurora.far contains note+ins without volume. Most are
+		     * after break positions, but one is played in order 16.
+		     * This is invalid and sets sets the volume to zero.
+		     * Volumes greater than 0x10 (f) also usually seem to be
+		     * treated as zero (OOB table read?).
+		     */
+		    event->vol = 1;
+		}
 
 		far_translate_effect(event, MSN(fxb), LSN(fxb), vol);
 	    }
@@ -442,6 +453,7 @@ static int far_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	mod->xxs[i].flg |= fih.loopmode ? XMP_SAMPLE_LOOP : 0;
 	mod->xxi[i].sub[0].vol = 0xff; /* fih.volume; */
+	mod->xxi[i].sub[0].pan = XMP_INST_NO_DEFAULT_PAN;
 	mod->xxi[i].sub[0].sid = i;
 
 	libxmp_instrument_name(mod, i, fih.name, 32);
