@@ -2,7 +2,6 @@
 #define LIBXMP_IFF_H
 
 #include "../hio.h"
-#include "../list.h"
 
 #define IFF_NOBUFFER 0x0001
 
@@ -23,21 +22,20 @@ struct iff_header {
 	char id[4];		/* IFF type identifier */
 };
 
-struct iff_info {
-	char id[4];
-	int (*loader)(struct module_data *, int, HIO_HANDLE *, void *);
-	struct list_head list;
-};
+LIBXMP_BEGIN_DECLS
+
+typedef int (*iff_loader)(struct module_data *, uint32, HIO_HANDLE *, void *);
 
 iff_handle libxmp_iff_new(void);
 int	libxmp_iff_load(iff_handle, struct module_data *, HIO_HANDLE *, void *);
 /* int libxmp_iff_chunk(iff_handle, struct module_data *, HIO_HANDLE *, void *); */
-int 	libxmp_iff_register(iff_handle, const char *,
-	int (*loader)(struct module_data *, int, HIO_HANDLE *, void *));
+int 	libxmp_iff_register(iff_handle, const char *, iff_loader);
 void 	libxmp_iff_id_size(iff_handle, int);
 void 	libxmp_iff_set_quirk(iff_handle, int);
 void 	libxmp_iff_release(iff_handle);
 /* int 	libxmp_iff_process(iff_handle, struct module_data *, char *, long,
 	HIO_HANDLE *, void *); */
 
+LIBXMP_END_DECLS
+
 #endif /* LIBXMP_IFF_H */
