@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2024 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2025 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -190,9 +190,13 @@ struct it_sample_header {
 	uint8 vit;		/* Vibrato waveform */
 };
 
+LIBXMP_BEGIN_DECLS
+
 int itsex_decompress8(HIO_HANDLE *src, uint8 *dst, int len,
 		      uint8 *tmp, int tmplen, int it215);
 int itsex_decompress16(HIO_HANDLE *src, int16 *dst, int len,
 		       uint8 *tmp, int tmplen, int it215);
 
+LIBXMP_END_DECLS
+
 #endif /* LIBXMP_LOADERS_IT_H */
