@@ -90,6 +90,7 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	int playseq_offset;
 	int bpm_on, bpmlen, med_8ch, hexvol;
 	int max_lines;
+	int tracker_ver;
 	int retval = -1;
 
 	LOAD_INIT();
@@ -384,7 +385,8 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	mod->trk = mod->pat * mod->chn;
 
-	mmd_tracker_version(m, ver, 0, expdata_offset ? &expdata : NULL);
+	tracker_ver = mmd_tracker_version(m, ver, 0, med_8ch,
+					  expdata_offset ? &expdata : NULL);
 
 	MODULE_INFO();
 
@@ -440,11 +442,6 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 				event->ins = pos[1] & 0x3f;
 
-				/* Decay */
-				if (event->ins && !event->note) {
-					event->f2t = FX_MED_HOLD;
-				}
-
 				event->fxt = pos[2];
 				event->fxp = pos[3];
 				mmd_xlat_fx(event, bpm_on, bpmlen,
@@ -459,6 +456,8 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	if (libxmp_med_new_module_extras(m) != 0)
 		goto err_cleanup;
 
+	MED_MODULE_EXTRAS(*m)->tracker_version = tracker_ver;
+
 	/*
 	 * Read and convert instruments and samples
 	 */
@@ -482,15 +481,20 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		}
 
 		for (i = 0; i < mod->ins && i < expdata.s_ext_entries; i++) {
-			int skip = expdata.s_ext_entrsz - 4;
+			int skip = expdata.s_ext_entrsz;
 
 			D_(D_INFO "sample %d expsmp_offset = 0x%08lx", i, hio_tell(f));
 
-			exp_smp[i].hold = hio_read8(f);
-			exp_smp[i].decay = hio_read8(f);
-			exp_smp[i].suppress_midi_off = hio_read8(f);
-			exp_smp[i].finetune = hio_read8(f);
-
+			if (expdata.s_ext_entrsz >= 2) {	/* MED 3.00 / OctaMED V1 */
+				exp_smp[i].hold = hio_read8(f);
+				exp_smp[i].decay = hio_read8(f);
+				skip -= 2;
+			}
+			if (expdata.s_ext_entrsz >= 4) {	/* MED 3.20 / OctaMED V2 */
+				exp_smp[i].suppress_midi_off = hio_read8(f);
+				exp_smp[i].finetune = hio_read8(f);
+				skip -= 2;
+			}
 			if (expdata.s_ext_entrsz >= 8) {	/* Octamed V5 */
 				exp_smp[i].default_pitch = hio_read8(f);
 				exp_smp[i].instr_flags = hio_read8(f);
