@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2025 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2026 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -28,6 +28,7 @@
 #include "med.h"
 #include "loader.h"
 #include "../med_extras.h"
+#include "../path.h"
 
 #ifdef DEBUG
 const char *const mmd_inst_type[] = {
@@ -154,7 +155,7 @@ void mmd_xlat_fx(struct xmp_event *event, int bpm_on, int bpmlen, int med_8ch,
 		 * left half of the argument determines the decay and the
 		 * right half the hold.
 		 */
-		event->fxt = event->fxp = 0;
+		event->fxt = FX_MED_HOLD;
 		break;
 	case 0x09:
 		/* SECONDARY TEMPO 09
@@ -449,7 +450,11 @@ static void mmd_load_instrument_common(
 
 		info->length = instr->length;
 
-		if (ver >= 2 && expdata->s_ext_entrsz >= 18) {	/* MMD2+ long repeat */
+		/* OctaMED Soundstudio always loads these values but only uses
+		 * them in mixing mode (MMD3); prior versions ignore them.
+		 * MED Soundstudio 2 always uses these fields when found in
+		 * MMD0-2 modules, but can only save MMD3 modules. */
+		if (ver >= 3 && expdata->s_ext_entrsz >= 18) {
 			info->rep = exp_smp->long_repeat;
 			info->replen = exp_smp->long_replen;
 		} else {
@@ -593,7 +598,7 @@ static int mmd_load_hybrid_instrument(HIO_HANDLE *f, struct module_data *m, int
 	mmd_set_default_pitch_note(xxi, exp_smp, ver);
 	sub = &xxi->sub[0];
 
-	sub->pan = 0x80;
+	sub->pan = XMP_INST_NO_DEFAULT_PAN;
 	sub->vol = info.enable ? sample->svol : 0;
 	sub->xpo = info.sampletrans;
 	sub->sid = smp_idx;
@@ -619,7 +624,7 @@ static int mmd_load_hybrid_instrument(HIO_HANDLE *f, struct module_data *m, int
 		if (j >= xxi->nsm || smp_idx >= mod->smp)
 			return -1;
 
-		sub->pan = 0x80;
+		sub->pan = XMP_INST_NO_DEFAULT_PAN;
 		sub->vol = info.enable ? 64 : 0;
 		sub->xpo = info.synthtrans;
 		sub->sid = smp_idx;
@@ -711,7 +716,7 @@ static int mmd_load_synth_instrument(HIO_HANDLE *f, struct module_data *m, int i
 		if (j >= xxi->nsm || smp_idx >= mod->smp)
 			return -1;
 
-		sub->pan = 0x80;
+		sub->pan = XMP_INST_NO_DEFAULT_PAN;
 		sub->vol = info.enable ? 64 : 0;
 		sub->xpo = info.synthtrans;
 		sub->sid = smp_idx;
@@ -753,7 +758,7 @@ static int mmd_load_sampled_instrument(HIO_HANDLE *f, struct module_data *m, int
         if (libxmp_med_new_instrument_extras(xxi) != 0)
                 return -1;
 	MED_INSTRUMENT_EXTRAS(*xxi)->hold = exp_smp->hold;
-	xxi->rls = 0xfff - (exp_smp->decay << 4);
+	MED_INSTRUMENT_EXTRAS(*xxi)->decay = exp_smp->decay;
 
 	xxi->nsm = 1;
 	if (libxmp_alloc_subinstrument(mod, i, 1) < 0)
@@ -764,7 +769,7 @@ static int mmd_load_sampled_instrument(HIO_HANDLE *f, struct module_data *m, int
 	sub = &xxi->sub[0];
 
 	sub->vol = info.enable ? sample->svol : 0;
-	sub->pan = 0x80;
+	sub->pan = XMP_INST_NO_DEFAULT_PAN;
 	sub->xpo = info.sampletrans;
 	sub->sid = smp_idx;
 	sub->fin = exp_smp->finetune << 4;
@@ -852,7 +857,7 @@ static int mmd_load_iffoct_instrument(HIO_HANDLE *f, struct module_data *m, int
 		return -1;
 
 	MED_INSTRUMENT_EXTRAS(*xxi)->hold = exp_smp->hold;
-	xxi->rls = 0xfff - (exp_smp->decay << 4);
+	MED_INSTRUMENT_EXTRAS(*xxi)->decay = exp_smp->decay;
 
 	xxi->nsm = num_oct;
 	if (libxmp_alloc_subinstrument(mod, i, num_oct) < 0)
@@ -866,7 +871,7 @@ static int mmd_load_iffoct_instrument(HIO_HANDLE *f, struct module_data *m, int
 		sub = &xxi->sub[j];
 
 		sub->vol = info.enable ? sample->svol : 0;
-		sub->pan = 0x80;
+		sub->pan = XMP_INST_NO_DEFAULT_PAN;
 		sub->xpo = info.sampletrans - 12;
 		sub->sid = smp_idx;
 		sub->fin = exp_smp->finetune << 4;
@@ -989,7 +994,7 @@ int mmd_load_instrument(HIO_HANDLE *f, struct module_data *m, int i, int smp_idx
 int med_load_external_instrument(HIO_HANDLE *f, struct module_data *m, int i)
 {
 	struct xmp_module *mod = &m->mod;
-	char path[XMP_MAXPATH];
+	struct libxmp_path sp;
 	char ins_name[32];
 	HIO_HANDLE *s = NULL;
 
@@ -1001,10 +1006,13 @@ int med_load_external_instrument(HIO_HANDLE *f, struct module_data *m, int i)
 		mod->xxs[i].flg & XMP_SAMPLE_LOOP ? 'L' : ' ',
 		mod->xxi[i].sub[0].vol);
 
-	if (!libxmp_find_instrument_file(m, path, sizeof(path), ins_name))
+	libxmp_path_init(&sp);
+	if (libxmp_find_instrument_file(m, &sp, ins_name) != 0)
 		return 0;
 
-	if ((s = hio_open(path, "rb")) == NULL) {
+	s = hio_open(sp.path, "rb");
+	libxmp_path_free(&sp);
+	if (s == NULL) {
 		return 0;
 	}
 
@@ -1069,20 +1077,32 @@ void mmd_info_text(HIO_HANDLE *f, struct module_data *m, int offset)
 /* Determine an approximate tracker version from an MMD module since, unlike
  * MED4, they don't store any useful tracker information. If expdata is not
  * present in the module, it should be passed as NULL.
+ *
+ * Other untested bits that may eventually be useful:
+ * - OctaMED 3.00 changed the order the block array and sample array are
+ *   written in. This might be more foolproof than the song name.
+ * - InstrExt::finetune was "pad0" in OctaMED 2, should be non-zero only in 3+?
+ * - i_ext_entrsz: MED 3.20 + OctaMED 2.00 (already tested by s_ext_entrsz)
+ * - rgbtable: MED 3.20 + OctaMED 2.00 (already tested by s_ext_entrsz)
+ *   transition.med has s_ext_entrsz=4 but no rgbtable, though?
+ * - channel_split/n_info was added by OctaMED 2.00 (OctaMED only)? only saved
+ *   if the module was edited in score mode?
  */
 int mmd_tracker_version(struct module_data *m, int mmdver, int mmdc,
-	struct MMD0exp *expdata)
+			int med_8ch, struct MMD0exp *expdata)
 {
-	struct xmp_module *mod = &m->mod;
 	int soundstudio = 0;
 	int medver = 0;
 	int s_ext_entrsz = 0;
+	uint32 songnamelen = 0;
 	int mmdch = '0' + mmdver;
 
 	if (expdata) {
 		D_(D_INFO "expdata.s_ext_entrsz = %d", expdata->s_ext_entrsz);
 		D_(D_INFO "expdata.i_ext_entrsz = %d", expdata->i_ext_entrsz);
+		D_(D_INFO "expdata.songnamelen  = %u", (unsigned)expdata->songnamelen);
 		s_ext_entrsz = expdata->s_ext_entrsz;
+		songnamelen = expdata->songnamelen;
 	} else {
 		D_(D_INFO "expdata = NULL");
 	}
@@ -1102,15 +1122,22 @@ int mmd_tracker_version(struct module_data *m, int mmdver, int mmdc,
 		medver = MED_VER_OCTAMED_500;
 	} else if (mmdver >= 2) {
 		medver = MED_VER_OCTAMED_500;
+	} else if (songnamelen > 0) {		/* Added in OctaMED 3.00 */
+		medver = MED_VER_OCTAMED_300;
 	} else if (mmdver >= 1) {
-		medver = MED_VER_OCTAMED_400;
-	} else if (mod->chn > 4) {
-		medver = MED_VER_OCTAMED_200;
+		medver = MED_VER_OCTAMED_300;
 	} else if (s_ext_entrsz > 2) {		/* s_ext_entrsz == 4 */
-		medver = MED_VER_320;
+		if (med_8ch) {
+			medver = MED_VER_OCTAMED_200;
+		} else {
+			medver = MED_VER_320;
+		}
 	} else if (expdata != NULL) {		/* s_ext_entrsz == 2 */
-		/* MED 3.00 and 3.10 i_ext_entrsz always 0? */
-		medver = MED_VER_300;
+		if (med_8ch) {
+			medver = MED_VER_OCTAMED_100;
+		} else {
+			medver = MED_VER_300;
+		}
 	} else {
 		medver = MED_VER_210;
 	}
