@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2022 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2026 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -79,7 +79,7 @@ struct local_data {
 	int has_ptdt;
 };
 
-static int get_info(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
+static int get_info(struct module_data *m, uint32 size, HIO_HANDLE *f, void *parm)
 {
 	struct xmp_module *mod = &m->mod;
 	struct local_data *data = (struct local_data *)parm;
@@ -123,16 +123,17 @@ static int get_info(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	return 0;
 }
 
-static int get_cmnt(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
+static int get_cmnt(struct module_data *m, uint32 size, HIO_HANDLE *f, void *parm)
 {
-	D_(D_INFO "Comment size: %d", size);
+	D_(D_INFO "Comment size: %u", (unsigned)size);
 
 	return 0;
 }
 
-static int get_ptdt(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
+static int get_ptdt(struct module_data *m, uint32 size, HIO_HANDLE *f, void *parm)
 {
 	struct local_data *data = (struct local_data *)parm;
+	long pos, pos2;
 
 	/* Sanity check */
 	if(data->has_ptdt) {
@@ -140,8 +141,30 @@ static int get_ptdt(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	}
 	data->has_ptdt = 1;
 
+	pos = hio_tell(f);
 	ptdt_load(m, f, 0);
 
+	/* Check total loaded length.
+	 * NOTE: some Protracker 3.6 files lie about the length of the PTDT
+	 * chunk due to what appears to be a bug--files with samples of length
+	 * 64k or greater will be undercounted in the chunk length by 65536. */
+	pos2 = hio_tell(f);
+	if (pos < pos2) {
+		int adjust = 0;
+		int i;
+
+		for (i = 0; i < m->mod.smp; i++) {
+			if (m->mod.xxs[i].len >= 65536) {
+				adjust += 65536;
+			}
+		}
+		if (pos2 - pos - adjust < 0 ||
+		    pos2 - pos - adjust != (int64)size) {
+			D_(D_WARN "PTDT length check failed: "
+			   "length %ld - adjust %d != %u (report this)",
+			   pos2 - pos, adjust, (unsigned)size);
+		}
+	}
 	return 0;
 }
 
@@ -253,7 +276,7 @@ static int ptdt_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 		mod->xxi[i].sub[0].fin = (int8)(mh.ins[i].finetune << 4);
 		mod->xxi[i].sub[0].vol = mh.ins[i].volume;
-		mod->xxi[i].sub[0].pan = 0x80;
+		mod->xxi[i].sub[0].pan = XMP_INST_NO_DEFAULT_PAN;
 		mod->xxi[i].sub[0].sid = i;
 		mod->xxi[i].rls = 0xfff;
 
