@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2022 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2026 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -114,7 +114,6 @@ static int stx_test(HIO_HANDLE * f, char *t, const int start)
 }
 
 #define FX_NONE 0xff
-
 static const uint8 fx[11] = {
 	FX_NONE,
 	FX_SPEED,
@@ -133,7 +132,7 @@ static int stx_load(struct module_data *m, HIO_HANDLE *f, const int start)
 {
 	struct xmp_module *mod = &m->mod;
 	int c, r, i, broken = 0;
-	struct xmp_event *event = 0, dummy;
+	struct xmp_event *event = NULL, dummy;
 	struct stx_file_header sfh;
 	struct stx_instrument_header sih;
 	uint8 n, b;
@@ -272,7 +271,7 @@ static int stx_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			mod->xxs[i].lpe = 0;
 		mod->xxs[i].flg = mod->xxs[i].lpe > 0 ? XMP_SAMPLE_LOOP : 0;
 		mod->xxi[i].sub[0].vol = sih.vol;
-		mod->xxi[i].sub[0].pan = 0x80;
+		mod->xxi[i].sub[0].pan = XMP_INST_NO_DEFAULT_PAN;
 		mod->xxi[i].sub[0].sid = i;
 		mod->xxi[i].nsm = 1;
 
