@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2024 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2026 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -254,7 +254,7 @@ static int ult_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 /* TODO: Add logarithmic volume support */
 	mod->xxi[i].sub[0].vol = uih.volume;
-	mod->xxi[i].sub[0].pan = 0x80;
+	mod->xxi[i].sub[0].pan = XMP_INST_NO_DEFAULT_PAN;
 	mod->xxi[i].sub[0].sid = i;
 
 	libxmp_instrument_name(mod, i, uih.name, 24);
