@@ -161,6 +161,8 @@ struct channel_data {
 	} retrig;
 
 	struct {
+#define TREMOR_SUPPRESS	0x40	/* Ignore tremor state until next update (FT2) */
+#define TREMOR_ON	0x80
 		uint8 up,down;	/* Tremor value */
 		uint8 count;	/* Tremor counter */
 		uint8 memory;	/* Tremor memory */
@@ -268,6 +270,7 @@ struct channel_data {
 
 	struct xmp_event delayed_event;
 	int delayed_ins;	/* IT save instrument emulation */
+	int key_memory;		/* Previous key (XM) */
 
 	int info_period;	/* Period */
 	int info_pitchbend;	/* Linear pitchbend */
@@ -276,11 +279,12 @@ struct channel_data {
 	int info_finalpan;	/* Final pan including envelopes */
 };
 
+LIBXMP_BEGIN_DECLS
 
 void	libxmp_process_fx	(struct context_data *, struct channel_data *,
-				 int, struct xmp_event *, int);
+				 int, const struct xmp_event *, int);
 void	libxmp_filter_setup	(int, int, int, int*, int*, int *);
-int	libxmp_read_event	(struct context_data *, struct xmp_event *, int);
+int	libxmp_read_event	(struct context_data *, const struct xmp_event *, int);
 
 void	libxmp_process_pattern_loop	(struct context_data *,
 	struct flow_control *f, int, int, int);
@@ -291,4 +295,10 @@ void	libxmp_process_pattern_break	(struct context_data *,
 void	libxmp_process_line_jump	(struct context_data *,
 	struct flow_control *f, int, int);
 
+/* For virt_pastnote() */
+void	libxmp_player_set_release	(struct context_data *, int);
+void	libxmp_player_set_fadeout	(struct context_data *, int);
+
+LIBXMP_END_DECLS
+
 #endif /* LIBXMP_PLAYER_H */
