@@ -11,6 +11,8 @@
 #define VIRT_ACTIVE		0x100
 #define VIRT_INVALID		-1
 
+LIBXMP_BEGIN_DECLS
+
 int	libxmp_virt_on		(struct context_data *, int);
 void	libxmp_virt_off		(struct context_data *);
 int	libxmp_virt_mute	(struct context_data *, int, int);
@@ -37,4 +39,6 @@ void	libxmp_virt_release	(struct context_data *, int, int);
 void	libxmp_virt_reverse	(struct context_data *, int, int);
 int	libxmp_virt_getroot	(struct context_data *, int);
 
+LIBXMP_END_DECLS
+
 #endif /* LIBXMP_VIRTUAL_H */
